# Copyright(c) 2000-2005 Association of Universities for Research in Astronomy, Inc.

procedure gemhedit(image,field,value,comment)

# Edit FITS header keyword:
# Adding comment if keyword does not already exist
# If keyword already exists, hedit is used and comment ignored
# Should be safe and never return status=1 unless image does not exist
#
# Version  May 31, 2001   IJ, v1.2 release
#          Mar  3, 2002   IJ, v1.3 release
#          Sept 20,2002   IJ, v1.4 release
#          Feb 3, 2003    IJ  fix segmentation violation bug under Linux7.3
#	   Aug 11, 2003   KL, Added 'addonly-' parameter to hedit calls

char image      {prompt="Input image"}
char field      {prompt="Keyword name"}
char value      {prompt="Value"}
char comment    {prompt="Comment to add for keyword"}
int  status     {0,prompt="Exit status"}

begin

    char l_image, l_field, l_value, l_comment, ll_value
    bool l_flstring
    real l_test
    char l_keycheck
    
    l_image = image ; l_field=field ; l_value=value ; l_comment=comment

    # Make sure the field name is in upper case
    print (l_field) | translit ("STDIN","a-z","A-Z",del-,coll-) | scan (l_field)

    # Initialize param
    status = 0
    cache ("imgets")

    if (no==imaccess(l_image)) {
        print("ERROR - GEMHEDIT: Image "//l_image//" not found")
        status=1
        bye
    }

    # Cut keyword to eight if it is longer
    if (strlen(l_field)>8)
        l_field=substr(l_field,1,8)

    # check if keyword already exists, use hedit if keyword already exists
    l_keycheck=""
    hselect(l_image,l_field,yes) | scan(l_keycheck)
    imgets(l_image,l_field, >>& "dev$null")

    if (imgets.value!="0" || l_keycheck!="")
        hedit(l_image,l_field,l_value,add-,addonly-,del-,verify-,show-,update+)
    else { 

        # Check if the input value a number or a string, checks real and int
        l_test=-9999999999.
        printf("%e\n",l_value) | scan(l_test)

        # Not completely safe, but the only way to make this work
        # on exponential notation with large precision
        if (l_test==-9999999999.)
            l_flstring=yes
        else {
	    #Test for date.
	    # Must detect '-9' and '9.999e-9' as numbers but 9999-99-99 as date
	    if (( stridx("-",l_value) > 1 ) && ( stridx("e",l_value) == 0 ) &&
	        ( stridx("E",l_value) == 0 ))	    
                l_flstring=yes
            else
	        l_flstring=no
	}

doneset:
        if (l_flstring) {
            if (strlen(l_value)>66) {
                l_value=substr(l_value,1,66)
                l_comment=""
            }

            if (strlen(l_value)<=18) {
                if (strlen(l_comment)>46)
                    l_comment=substr(l_comment,1,46)
		    
                printf("%-8s= \'%-18s\' / %-s\n",l_field,l_value,l_comment) | \
                    mkheader(l_image,"STDIN",append+,verbose-)
            } else {
                if (strlen(l_comment)>80-(16+strlen(l_value)) && l_comment!="")
                    l_comment=substr(l_comment,1,80-(16+strlen(l_value)))
		    
                printf("%-8s= \'%-s\' / %-s\n",l_field,l_value,l_comment) | \
                    mkheader(l_image,"STDIN",append+,verbose-)
            }

        } else {
            if (strlen(l_comment)>46)
                l_comment=substr(l_comment,1,46)

            ll_value=l_value
            printf("%-8s= %20.10f / %-s\n",l_field,99.99,l_comment) | \
                mkheader(l_image,"STDIN",append+,verbose-)
            hedit(l_image,l_field,l_value,add-,addonly-,del-,ver-,show-,update+)
        }

        # Make sure iraf sees the last keyword added
        hedit(l_image,"junkgem","0",add+,addonly-,del-,ver-,show-,update+)
        hedit(l_image,"junkgem","",add-,addonly-,del+,ver-,show-,update+)

    } # end of using mkheader

end
