# Copyright(c) 2002-2005 Association of Universities for Research in Astronomy, Inc.

procedure gnscombine(inimages,offsets,outimage)

# Task to combine gnsskysub reduced images
#
# Version Jun 03, 2003   RA,KG,IJ First version tested w/ v1.4, not ready for 
#                        release the task has known bugs
# Version Aug 02, 2003   RA Known bugs fixed and task sent to Inger for approval

char inimages      {"",prompt="Input GMOS images or list"}
char offsets       {"",prompt="Text file with X,Y offsets"}
char outimage      {"",prompt="Output image"}
char logfile       {"",prompt="Logfile"}
char outcheckim    {"",prompt="Name of output check (non-crrej) image"}
char outmedsky     {"",prompt="Name of output median (for sky) image"}
bool verbose       {yes,prompt="Verbose"}
struct* scanfile   {"",prompt="Internal use only"}

begin

    char l_inimages, l_outimage, l_outcheckim, l_offsets, l_dir, l_outmedsky
    char l_logfile
    bool l_verbose
    char l_temp
    struct l_struct
    int l_test,i,ninp,noff,nbad,nsci,n_ext
    char img, filelist, templist, medframe, semifinal
    char temp1, temp2, temp3, temp4
    char inimg[200]
    char temp[200]
    real x,y,dx[200],dy[200]
    char tempWithSky[200]
    int maxfiles
    int shuffle, xbin, ybin
    int status

    # Localize global variables
    l_inimages=inimages ; l_offsets=offsets ; l_outimage=outimage
    l_outcheckim = outcheckim ; l_outmedsky = outmedsky ; l_verbose=verbose
    l_logfile = logfile

    maxfiles=200

    # Create any needed temporary file names  (more defined later)
    filelist = mktemp("tmpfile0")
    templist = ""
    medframe = ""
    semifinal = ""

    #-------------------------------------------------------------------
    # Test the logfile:

    if (l_logfile == "" || stridx(" ",l_logfile)>0) {
	l_logfile = gmos.logfile
	if (l_logfile == "" || stridx(" ",l_logfile)>0) {
            l_logfile = "gmos.log"
            printlog("WARNING - GNSCOMBINE: Both gnscombine.logfile and gmos.logfile fields are empty",
	        l_logfile,yes)
            printlog("                    Using default file gmos.log",
	        l_logfile,yes)
	}
    }


    #-------------------------------------------------------------------
    # Start logging

    date | scan(l_struct)
    printlog("----------------------------------------------------------------------------",
        l_logfile,l_verbose)
    printlog("GNSCOMBINE -- "//l_struct,l_logfile,l_verbose)
    printlog("",l_logfile,l_verbose)
    printlog("Input list    = "//l_inimages,l_logfile,l_verbose)
    printlog("Output list   = "//l_outimage,l_logfile,l_verbose)
    printlog("",l_logfile,l_verbose)


    #-------------------------------------------------------------------
    # Last chance to do fiddly stuff before things start seriously happening!
    cache("imgets")


    #-------------------------------------------------------------------
    # Load up input name list: @list, * and ?, comma separated
    if (l_inimages == "" || stridx(" ",l_inimages)>0) {
	printlog("ERROR - GNSCOMBINE: Input file not specified",l_logfile,yes)
	goto crash
    }

    # Test for @filelist
    if (substr(l_inimages,1,1) == "@" && !access(substr(l_inimages,2,strlen(l_inimages))) ) {
        printlog("ERROR - GNSCOMBINE: Input list "//substr(l_inimages,2,strlen(l_inimages))//" does not exist",
	    l_logfile,yes)
        goto crash
    }


    # parse wildcard and comma-separated lists
    if (substr(l_inimages,1,1)=="@")  {
        scanfile=substr(l_inimages,2,strlen(l_inimages))
        while (fscan(scanfile,l_temp) != EOF) {
	    files(l_temp, >> filelist)
        }
    } else {
        if (stridx(",",l_inimages)==0)
	    files(l_inimages, > filelist)
        else {
	    l_test=9999
	    while (l_test!=0) {
	        l_test=stridx(",",l_inimages)
	        if (l_test>0)
	            files(substr(l_inimages,1,l_test-1), >> filelist)
	        else
	            files(l_inimages, >> filelist)
	        l_inimages=substr(l_inimages,l_test+1,strlen(l_inimages))
	    }
        }
    }
    scanfile=""

    #--------------------------------------------------------------------
    # Define the input filename list

    ninp = 0
    nbad=0
    scanfile=filelist
    while (fscan(scanfile, img) != EOF) {

       # split off directory path
       fparse(img,verbose-)

       img=fparse.root
       l_dir=fparse.directory

       ninp = ninp+1
       if (ninp > maxfiles) {
            printlog("ERROR - GNSCOMBINE: Maximum number of input images exceeded",
	        l_logfile,yes)
            goto crash
       }
       inimg[ninp] = l_dir//img//".fits"  # now has path if relevant

    } # end while
    scanfile = ""

    # At the end of all this inimg[ninp] now contains the input images incl. 
    # the directory path
    #--------------------------------------------------------------------------
    # Define the offsets lists
    
    if (access (l_offsets) == no) {
        printlog ("ERROR - GNSCOMBINE: Offset file '"//l_offsets//"' \
            not found.", l_logfile, verbose+)
        goto crash
    }

    scanfile=l_offsets
    noff=1
    while (fscan(scanfile,x,y) != EOF){
	dx[noff]=x
	dy[noff]=y
	noff = noff + 1
    }
    if (ninp > maxfiles) {
	printlog("ERROR - GNSCOMBINE: Number of offsets does not correspond to number of images",
	    l_logfile,yes)
	goto crash
    }

    #--------------------------------------------------------------------------
    # Run a quick check to see if the input images have been gprepared
    # KNOWN BUG: l_logfile not defined, IJ
    i = 1
    while (i<=ninp) {
	imgets(inimg[i]//"[0]","NSCIEXT", >>& "dev$null")
	nsci=int(imgets.value)
	if (nsci==0) {
            printlog("ERROR - GNSCOMBINE: Keyword NSCIEXT not found in image.",
	        l_logfile,YES)
            printlog("ERROR - GNSCOMBINE: File "//inimg[i]//" is not gprepared.",
	        l_logfile,YES)
            printlog("ERROR - GNSCOMBINE: Please run gprepare on all input images.",
	        l_logfile,YES)
            goto crash
	}
	i=i+1
    }

    #--------------------------------------------------------------------------
    # Making the output image. This is where all the action really is!
    # Step 0. Copy files for templates
    printlog("GNSCOMBINE\n",l_logfile,yes)
    printlog("Creating temporary files\n",l_logfile,yes)
    i = 1
    while (i<=ninp) {
        temp[i]=mktemp("tmp")
        temp[i]=temp[i]//".fits"
        copy(inimg[i],temp[i])
        i=i+1
    }

    # Step 1. Shift images
    printlog("Shifting images\n",l_logfile,yes)
    i = 1
    while (i<=ninp) {
        for(n_ext=1;n_ext<=nsci;n_ext+=1) {
          imshift(input=temp[i]//"[SCI,"//str(n_ext)//"]",
              output=temp[i]//"[SCI,"//str(n_ext)//",overwrite]",
	      xshift=dx[i],yshift=dy[i],interp="nearest",shifts_file="",boundary_type="nearest",constant=0.)
        }
        i=i+1
    }

    # Step 2. Get median sky
    printlog("Getting median sky\n",l_logfile,yes)
    i = 1
    templist=mktemp("list")
    medframe=mktemp("med")
    medframe=medframe//".fits"
    if (l_outmedsky != "")
        medframe = l_outmedsky
    while (i<=ninp) {
        print(temp[i],>>templist)
        i=i+1
    }
    gemcombine(input="@"//templist,output=medframe,combine="median",
        nrejfile="",reject="none",offsets="none", logfile=l_logfile)
    printlog("",l_logfile,yes)

    # Shift the sky frame - this just makes it look pretty with gnsskysub 
    # doing A-B
    # The combined image may still have blotches in the +sky or -sky areas
    hselect(medframe//"[0]","NODPIX",yes) | scan(shuffle)
    printlog ("Making shifted sky for shuffle = "//shuffle,l_logfile,yes)
    # sci_ext hardwirded for now - task does not comply with coding standard
    hselect(medframe//"[SCI,1]","CCDSUM",yes) | \
        translit("STDIN",'"',"",delete+,collapse-) | scan(xbin,ybin)
    # Correct shuffle for bining in Y
    shuffle=shuffle/ybin
    for(n_ext=1;n_ext<=nsci;n_ext+=1) {
        imshift(input=medframe//"[SCI,"//str(n_ext)//"]",
            output=medframe//"[SCI,"//str(n_ext)//",overwrite]",xshift=0,
            yshift=shuffle,interp="nearest",shifts_file="",boundary_type="nearest",constant=0.)
    }

    # Step 3. Slice and dice
    printlog("Calling gnsskysub",l_logfile,yes)
    gnsskysub(inimages="@"//templist,fl_fixnc+)
    printlog("",l_logfile,yes)

    # Step 4. Add back the constant sky (so errors are right)
    printlog("Adding back constant sky so errors are computed correctly in the next step",
        l_logfile,yes)
    delete(templist,ver-, >>& "dev$null")
    i = 1
    while (i<=ninp) {
        tempWithSky[i]=mktemp("tmp")
        printlog("Adding median sky to "//"n"//temp[i]//" -> "//tempWithSky[i],
	    l_logfile,yes)
        gemarith(operand1="n"//temp[i],op="+",operand2=medframe,
	    result=tempWithSky[i], logfile=l_logfile)
        print(tempWithSky[i],>>templist)
        i=i+1
    }
    printlog("",l_logfile,yes)

    # Step 5. Now do the imcombine
    printlog("Running imcombine",l_logfile,yes)
    semifinal=mktemp("tmp")
    semifinal=semifinal//".fits"
    gemcombine(input="@"//templist,output=semifinal,reject="ccdclip",
        nrejfile="",lsigma=10,hsigma=10,grow=2,offsets="none", 
	logfile=l_logfile)

    # Step 6. Now remove the constant sky again
    printlog ("Subtracting median sky",l_logfile,yes)
    gemarith (operand1=semifinal,op="-",operand2=medframe,result=l_outimage,
        logfile=l_logfile)
    printlog ("Created cr-rej combined image "//l_outimage,l_logfile,yes)

    if (l_outmedsky != "") 
        printlog("Created median frame for sky "//l_outmedsky,l_logfile,yes)

    if (l_outcheckim == "") goto clean

    # Step7 Make check image (no rejection)
    printlog ("Making check [non-crrej] image "//l_outcheckim,l_logfile,yes)
    delete(templist,ver-, >>& "dev$null")
    i = 1
    while (i<=ninp) {
        print("n"//temp[i],>>templist)
        i=i+1
    }
    printlog("",l_logfile,yes)
    gemcombine(input="@"//templist,output=l_outcheckim,reject="none",
        nrejfile="",offsets="none", logfile=l_logfile)
    printlog("Created non-crrej combined image "//l_outcheckim,l_logfile,yes)

    goto clean

crash:
    # Exit with error subroutine
    status=1
    printlog("ERROR - GNSCOMBINE: Exit with error",l_logfile,yes)
    goto clean

clean:
    # clean up
    delete(filelist,ver-, >>& "dev$null")
    delete(templist,ver-, >>& "dev$null")
    if (l_outmedsky == "") delete (medframe,ver-, >>& "dev$null")
    delete(semifinal,ver-,>>& "dev$null")
    scanfile=""
    i = 1
    while (i<=ninp) {
	delete(temp[i],ver-,>>& "dev$null")
	delete("n"//temp[i],ver-,>>& "dev$null")
	delete(tempWithSky[i]//".fits",ver-,>>& "dev$null")
	i=i+1
    }
end
