# GNIRS data reduction script
#
# This data processing was done for data quality assessment by Gemini staff.
# The processing is not designed to give the best possible result.  The user
# of these data is encouraged to use the provided "final" images, and the
# processing steps described here, only as a guide and to re-reduce the
# data carefully to obtain the best possible reduction.
#
# Observation UT date: 2004 Apr 13
# Data filename prefix: S20040413S
#
# Comments from data processor:
#
# This is an example of a basic reduction of a cross-dispersed/32 
#   (R~2000) dataset.  The obslog entries included below (just before the 
#   processing steps) give a brief description of the data.
# This reduction assumes various calibration files exist in the caldir:
#    Flat: snS20040413S0456_flat.fits
#    S-dist: rnS20040413S0504.fits
#    Arc:  wtrnS20040413S0480.fits
#
# These files can be created with the XD32-Cal example.
#    
#
# This line points to the raw data 
set rawdir=/net/veronica/staging/gnirs/2004apr13/
# This line points to calibration data
set caldir=dgnirs$2004apr13/Basecalib/

# Load required tasks/packages
gemini
gnirs

#-----------setup for gnirs-----------
#set default parameters
#use gnirs defaults except where specified on command lines below
unlearn gemini
unlearn gemtools
unlearn gnirs

#define logfile and database
#to start from scratch, delete existing log file
delete 20040413_GS-2004A-SV-11.log
gnirs.logfile="20040413_GS-2004A-SV-11.log"
#set database dir
gnirs.database="20040413_GS-2004A-SV-11_database"
#to start from scratch, delete database entries if present
if (access("20040413_GS-2004A-SV-11_database")) delete 20040413_GS-2004A-SV-11_database/*

#load default header keywords for gnirs
nsheaders gnirs
set stdimage=imt1024

### SV 11 Obslog entries
#GS-2004A-SV-11-13 411-415 04:21 HD96252     ab5 XD 1.0/SCXD 32/2.208 SBC/XD 10/1/2   AM=1.17 
#GS-2004A-SV-11-7  423-429 04:40 HE1117-1735 ab6 XD 1.0/SCXD 32/2.208 SBC/XD 120/16/1 AM=1.21; dithered each ab pair slightly along slit for better CR corrections; lost guide on 426 but its sky obs; lost again on 427, no source in 429; Use 423-426
#GS-2004A-SV-11-29 442-445 06:15 HE1117-1735 ab6 XD 1.0/SCXD 32/2.208 SBC/XD 120/16/1 AM=1.61; repeated 2 ab pairs, and target IS there.  Focus may be 67000 instead of 53000.

#--------create lists--------
del *.lis,all.lis
gemlist S20040413S 412-415 > obj1.lis 
gemlist S20040413S 423-426 > obj2a.lis
gemlist S20040413S 442-445 > obj2b.lis
concat obj*.lis > all.lis

#--------reduce objects-------
# shift from cal
imdel n@all.lis
nsprepare @all.lis rawpath=rawdir$ shiftx=2.8

# sky range for obj1
del phu.lis
gemextn n@obj1.lis proc=expand index=0 > phu.lis
hselect @phu.lis $I,ut yes >> 20040413_GS-2004A-SV-11.log
#frames are ~30 secs apart

imdel rn@obj1.lis
nsreduce n@obj1.lis fl_nscut+ fl_nsappwave- fl_process_cut+ fl_dark- \
  fl_sky+ skyrange=45 fl_flat+ flatim=caldir$rnS20040413S0456_flat.fits


# sky range for obj2
del phu.lis
gemextn n@obj2a.lis proc=expand index=0 > phu.lis
hselect @phu.lis $I,ut yes >> 20040413_GS-2004A-SV-11.log
#frames are ~140 secs apart
del phu.lis
gemextn n@obj2b.lis proc=expand index=0 > phu.lis
hselect @phu.lis $I,ut yes >> 20040413_GS-2004A-SV-11.log
#frames are ~140 secs apart

# to look at time and spatial offsets, can also use gemoffsetlist:
#gemoffsetlist n@obj2a.lis nS20040413S0423 1 INDEF "dev$null" "dev$null" verb+
# look at data on obj2a.lis
# no data in S0424 or S0426?!  what happened?
#nscut nS20040413S0442
#imexam snS20040413S0442[sci,1]
# nada! squit! nuttin'!
# look at data on obj2b.lis
# no data in S0442 or S044 this time?!

# so separate into object and sky lists
del obj2aobj.lis,obj2asky.lis
gemlist S20040413S 424-426x2 > obj2asky.lis
gemlist S20040413S 423-426x2 > obj2aobj.lis

imdel rn@obj2aobj.lis
nsreduce n@obj2aobj.lis skyimages=n@obj2asky.lis \
  fl_nscut+ fl_nsappwave- fl_process_cut+ fl_dark- \
  fl_sky+ skyrange=200 nodsize=INDEF fl_flat+ \
  flatim=caldir$rnS20040413S0456_flat.fits

del obj2bobj.lis,obj2bsky.lis
gemlist S20040413S 442-445x2 > obj2bsky.lis
gemlist S20040413S 443-445x2 > obj2bobj.lis

imdel rn@obj2bobj.lis
nsreduce n@obj2bobj.lis skyimages=n@obj2bsky.lis \
  fl_nscut+ fl_nsappwave- fl_process_cut+ fl_dark- \
  fl_sky+ skyrange=200 nodsize=INDEF fl_flat+ \
  flatim=caldir$rnS20040413S0456_flat.fits

# transform 
# need to copy ref files from Basecalib
imdel wrnS20040413S0480
copy caldir$wrnS20040413S0480.fits .
imdel rnS20040413S0504
copy caldir$rnS20040413S0504.fits .

imdel trn@obj1.lis
nstransform rn@obj1.lis lamp=wrnS20040413S0480 \
  sdist=rnS20040413S0504 fl_align+ database=caldir$GS-CAL20040413_database
#nxdisplay trnS20040413S0412 1

imdel trn@obj2aobj.lis,trn@obj2bobj.lis
nstransform rn@obj2aobj.lis,rn@obj2bobj.lis lamp=wrnS20040413S0480 \
  sdist=rnS20040413S0504 fl_align+ database=caldir$GS-CAL20040413_database

# combine
imdel trnS20040413S0412_comb
nscombine trn@obj1.lis
#nxdisplay trnS20040413S0412_comb 1

imdel trnS20040413S0423_comb
nscombine trn@obj2aobj.lis,trn@obj2bobj.lis
#nxdisplay trnS20040413S0423_comb 2

# misaligned during combination
#display rnS20040413S0423[sci,1] 1
#display rnS20040413S0425[sci,1] 2
#display rnS20040413S0443[sci,1] 3
#display rnS20040413S0445[sci,1] 4
# look ok in raw, but "b" group shifted by 33 pixels during nstransform!
#gemoffsetlist n@obj2aobj.lis,n@obj2bobj.lis nS20040413S0423 1 INDEF \
#  "dev$null" "dev$null" verb+
# has 5 arcsec shift (5/0.15=33 pixels)
#del phu.lis
#gemextn n@obj2aobj.lis,n@obj2bobj.lis index="0" > phu.lis
#hselect @phu.lis $I,RA,DEC yes
#hselect @phu.lis $I,XOFFSET,YOFFSET,RAOFFSET,DECOFFSE yes
#hselect @phu.lis $I,CRPIX1,CRVAL1 yes
#hselect @phu.lis $I,CRPIX2,CRVAL2 yes
# "b" group have offset, according to headers, but this isn't apparent in 
# data.
# so let's re-run transform without alignment, then stack without shifting

imdel trn@obj2aobj.lis,trn@obj2bobj.lis
nstransform rn@obj2aobj.lis,rn@obj2bobj.lis lamp=wrnS20040413S0480 \
  sdist=rnS20040413S0504 fl_align- database=caldir$GS-CAL20040413_database

# gemcombine doesn't handle prefix@file expansion correctly, so use 
# gemextn first
del obj2tmp.lis
gemextn trn@obj2aobj.lis,trn@obj2bobj.lis proc="none" > obj2tmp.lis
gemcombine @obj2tmp.lis obj2 logfile=gnirs.logfile
#nxdisplay obj2 1
# finally something that looks vaguely ok

# extract
imdel xtrnS20040413S0412_comb
nsextract trnS20040413S0412_comb

imdel xobj2
nsextract obj2

# telluric correction
imdel axtrnS20040413S0412_comb
nstelluric xtrnS20040413S0412_comb caldir$xtrnS20040413S0386_comb \
  high_rej=5 fitorder=15 thresh=0.001

imdel axobj2
nstelluric xobj2 caldir$xtrnS20040413S0386_comb \
  high_rej=5 fitorder=15 thresh=0.001

#----clean up
delete tmp*,*lis
imdel nS20040413S????.fits
imdel rnS20040413S????.fits
imdel trnS20040413S????.fits
imdel s*nS20040413S????.fits
