# Copyright(c) 2004-2005 Association of Universities for Research in Astronomy, Inc.
#
# GNIRS data reduction script
#
# This data processing was done for data quality assessment by Gemini staff.
# The processing is not designed to give the best possible result.  The user
# of these data is encouraged to use the provided "final" images, and the
# processing steps described here, only as a guide and to re-reduce the
# data carefully to obtain the best possible reduction.
#
# Observation UT date: 2004 Jun 13
# Data processor: Bernadette Rodgers
# Final Data reduction date:   2004 Aug 17
# Data filename prefix: S20040613S
#
# Last revised: 2005 Feb 28, Patrick Ingraham
#
# Comments from data processor:
#
#  This is an example of a data processing script for longslit medium resolution
#  (110 l/mm grating) data.  A brief description of the data is given by the 
#  obslog entry included now.  The script assumes that the calibrations have 
#  already been processed and the files:
#       rnS20040613S0172_flat.fits
#       wrnS20040613S0182.fits and
#       GS-CAL20040613_database/idwrnS20040613S0182_SCI_1_
#  exist in the caldir$ directory.  These can be created with the LS110-Cal example.
#
#
# This line points to the raw data 
set rawdir=/net/veronica/staging/gnirs/2004jun13/
# This line points to the calibration data 
set caldir=/staging/dataproc/gnirs/2004jun13/Basecalib/

# Load required tasks/packages 
gemini
gnirs

unlearn gemini
unlearn gemtools
unlearn gnirs

#setup for gnirs
delete 20040613_GS-2004A-SV-15.log
gnirs.logfile="20040613_GS-2004A-SV-15.log"

#set database dir
gnirs.database="20040613_GS-2004A-SV-15_database"

#to start from scratch, delete database entries if present
if (access("20040613_GS-2004A-SV-15_database"))
    delete 20040613_GS-2004A-SV-15_database/*
;

#set default parameters
nsheaders gnirs
set stdimage=imt1024

###Start processing
#GS-2004A-SV-15-14  117-123 06:10   GX349+2      none    H       acq        --/--        SB/acq     10/1/1         Acq.
#GS-2004A-SV-15-15  124-155 06:10   GX349+2      abcb    K       0.6       110/2.2       SB/Mir     120/16/1       AM=1.20-1.58, multiple objects in slit; readnoise looks high?; break between 140 and 141; completed 8 loops (of 15)
#GS-2004A-SV-15-14  156-157 08:26   GX349+2      none    H       acq        --/--        SB/acq     10/1/1         checking slit position

#-----Create lists-----
delete *.lis
gemlist S20040613S 124-155 > obj.lis

#-----reduce object-----
#shift for mdf from cross-correlation with calibration files
imdel n@obj.lis ver-
nsprepare @obj.lis rawpath=rawdir$ shiftx=25.

#check sky range
del phu.lis
!echo "--------------------------------------------" >> 20040613_GS-2004A-SV-15.log
gemextn n@obj.lis proc=expand index=0 > phu.lis
hselect @phu.lis $I,ut yes >> 20040613_GS-2004A-SV-15.log
hselect @phu.lis $I,ut yes # to show output
!echo "--------------------------------------------" >> 20040613_GS-2004A-SV-15.log
#frames are ~140 secs apart, use in skyrange parameter for nsreduce

#reduce - note the skysection which is carefully tailored to reduce skylines
imdel rn@obj.lis
nsreduce n@obj.lis fl_nscut+ fl_dark- fl_sky+ fl_nsapp- skyrange=160 fl_flat+ \
  flatim=caldir$rnS20040613S0172_flat skysec=[400:500,787:789]

#transform (wavelength only)
#need to copy reference files from cal dir
imdel wrnS20040613S0182.fits
copy caldir$wrnS20040613S0182.fits .
imdel trn@obj.lis
nstransform rn@obj.lis lamp=wrnS20040613S0182 \
  database=caldir$GS-CAL20040613_database/

#combine...
imdel trnS20040613S0124_comb
nscombine trn@obj.lis

#and extract.
# this has to be interactive because there are several emission sources,
# apparently
imdel xtrnS20040613S0124_comb
nsextract trnS20040613S0124_comb fl_inter+ fl_trace+

#delete extra files
delete *lis
delete tmp*
imdel nS20040613S????.fits
imdel rnS20040613S????.fits
imdel trnS20040613S????.fits

