# Copyright(c) 2004-2005 Association of Universities for Research in Astronomy, Inc.
#
# GNIRS data reduction script
#
# This data processing was done for data quality assessment by Gemini staff.
# The processing is not designed to give the best possible result.  The user
# of these data is encouraged to use the provided "final" images, and the
# processing steps described here, only as a guide and to re-reduce the
# data carefully to obtain the best possible reduction.
#
# Observation UT date: 2004 Jun 15
# Data processor:
# Data reduction date:   2004 July 16
# Data filename prefix: S20040615S
#
# Last revised: 2005 Feb 28, Patrick Ingraham
#
# Comments from data processor:
#
# This is an example of basic data reduction for longslit moderate (R~1700) 
#  resolution data of an extended source.  This case is different from the other 
#  examples in that the data were taken with an obj-sky dither pattern, i.e., 
#  only 50% of data are taken on-source (as opposed to dithering along the slit).
#  A brief summary of the data is provided by the Obslog entries, included below.
#  This reduction assumes the presence of the following files in the caldir
#  directory:
# 	Flat: caldir$rnS20040615S0186_flat.fits 
# 	Arc: caldir$wrnS20040615S0195.fits 
# 	S-dist: caldir$snS20040615S0145_stack_sdist.fits
#
#  These files can be created with the LS110-Cal example.
#

# This line points to the raw data 
set rawdir=/net/veronica/staging/gnirs/2004jun15/
# This line points to the reduced calibrations directory
set caldir=/net/josie/staging/dataproc/gnirs/2004jun15/Basecalib/

#Load required tasks/packages
gemini
gemtools
gnirs

#-----------setup for gnirs----------
#use gnirs defaults except where specified on command lines below
unlearn gemini
unlearn gemtools
unlearn gnirs

#set logfile
#delete 20040615_GS-2004A-SV-8.log
gnirs.logfile="20040615_GS-2004A-SV-8.log"

#set database dir
gnirs.database="20040615_GS-2004A-SV-8_database"

#to start from scratch, delete database entries if present
delete 20040615_GS-2004A-SV-8_database/*

# load header keywords
nsheaders gnirs

# set display
set stdimage=imt1024

#-------Obslog entry----------------
#GS-2004A-SV-8-5    156-183 02:39   NGC5128      abba     K     0.3        32/2.208     SB/mir     120/16/1 

#------create lists---------
#need seperate files for object and sky
del obj.lis,sky.lis
gemlist S20040615S 156,159,160,163,164,167,168,171,172,175,176,179,180,183 > obj.lis
gemlist S20040615S 157,158,161,162,165,166,169,170,173,174,177,178,181,182 > sky.lis

#first prepare all
#shift taken from calibration example, where measured with cross-corrlen
imdel n@obj.lis,n@sky.lis ver- >& dev$null
nsprepare @obj.lis,@sky.lis rawpath=rawdir$ shiftx=3.3

#reduce object frames; all images in the supplied sky list will be combined 
#and subtracted from all input frames.
del phu.lis
!echo "--------------------------------------------" >> 20040615_GS-2004A-SV-8.log
gemextn n@obj.lis proc="expand" index="0" > phu.lis
!echo "Exposure Time Stamps for skyrange parameter" >> 20040615_GS-2004A-SV-8.log
hselect @phu.lis "$I,UT" yes >> 20040615_GS-2004A-SV-8.log
!echo "--------------------------------------------" >> 20040615_GS-2004A-SV-8.log

imdel rn@obj.lis ver- >& dev$null
nsreduce n@obj.lis fl_nscut+ fl_dark- fl_sky+ skyimages=n@sky.lis \
  fl_flat+ flatim=caldir$rnS20040615S0186_flat.fits skyrange=180

#all source frames are at the same position, no need to use nscomb
imdel rnS20040615S0156_stack.fits
nsstack rn@obj.lis

#now transform final 2-D image
# - nstransform can apply both wavelength and s-distortion transformations.
# - need local copy of fields used for calibration
imdel wrnS20040615S0195
copy caldir$wrnS20040615S0195.fits .
imdel trnS20040615S0156_stack.fits
nstransform rnS20040615S0156_stack.fits lamp=wrnS20040615S0195 \
  database=caldir$GS-CAL20040615_database/ 

#extract data
imdel xtrnS20040615S0156_stack.fits
nsextract trnS20040615S0156_stack.fits
# s/n ~6
#splot xtrnS20040615S0156_stack[sci,1]

#telluric correction 
imdel axtrnS20040615S0156_stack
nstelluric xtrnS20040615S0156_stack xtrnS20040615S0145_comb \
  thresh=0.01 high=5 fitord=15 
#splot axtrnS20040615S0156_stack[sci,1]
#specplot xtrnS20040615S0156_stack[sci,1],axtrnS20040615S0156_stack[sci,1],\
#caldir$xtrnS20040615S0145_comb[sci,1]

# clean up
del tmp*,*.lis
# keep only final stacks, transformed and extracted
imdel nS20040615S????.fits,rnS20040615S????.fits

