# Copyright(c) 2004-2005 Association of Universities for Research in Astronomy,
Inc.
#
# GNIRS data reduction script
#
# This data processing was done for data quality assessment by Gemini staff.
# The processing is not designed to give the best possible result.  The user
# of these data is encouraged to use the provided "final" images, and the
# processing steps described here, only as a guide and to re-reduce the
# data carefully to obtain the best possible reduction.
#
# Observation UT date: 2004 Jun 13
# Data processor: Bernadette Rodgers
# Final Data reduction date:   2004 Aug 17
# Data filename prefix: S20040613S
#
# Last revised: 2005 Feb 28, Patrick Ingraham
#
# Comments from data processor:
#
# Brief data description --
#  This is an example of a basic calibration reduction for longslit/110 
#   (R~6000) dataset with central wavelength=2.2microns.  The data consists 
#   of GCAL flats, an arc and a telluric standard.  
#
#   The obslog entries are included below just before the processing steps.
#

# This line points to the raw data 
set rawdir=/net/veronica/staging/gnirs/2004jun13/

# Load required tasks/packages 
gemini
gnirs

#-----------setup for gnirs-----------
#set default parameters
#use gnirs defaults except where specified on command lines below
unlearn gemini
unlearn gemtools
unlearn gnirs

#define logfile and database
delete GS-CAL20040613.log
gnirs.logfile="GS-CAL20040613.log"

#set database dir
gnirs.database="GS-CAL20040613_database"

#to start from scratch, delete database entries if present
if (access("GS-CAL20040613_database"))
    delete GS-CAL20040613_database/*
;

#load default header keywords for gnirs
nsheaders gnirs
set stdimage=imt1024


###Start processing

###SV-15 Cals###
#GS-CAL20040613-8   161-168 08:35   HIP89399     abba    K       0.6       110/2.2       SB/mir      2/1/5          SV-15 std
#GS-CAL20040613-10  169-180 08:50   GCAL IR      none    K       0.6       110/2.2       SB/mir      1/1/1          SV-15 flats; use 171-180
#GS-CAL20040613-11  181-183         GCAL Ar      none    K       0.6       110/2.2       SB/mir      5,10/1/1       SV-15 arcs & pinhole(#183); 184&185 are eng. images 

#-----Create lists-----
delete *.lis
gemlist S20040613S 197-215 > dark.lis
gemlist S20040613S 171-180 > flat.lis
gemlist S20040613S 182 > arc.lis
gemlist S20040613S 183 > pinhole.lis
gemlist S20040613S 161-168 > telluric.lis

concat dark.lis,flat.lis,arc.lis,pinhole.lis,telluric.lis >> all.lis

#-----prepare everything-----
#this way, all mdfs shifted by same amount
imdel n@all.lis
nsprepare @all.lis rawpath=rawdir$ shiftx=INDEF shifty=INDEF
# shift is 25 (needed for processing observations)

#-----make flats-----
#first check flats
delete sciflat.lis
!echo "--------------------------------------------" >> GS-CAL20040613.log
gemextn n@flat.lis proc="expand" extname="SCI" > sciflat.lis
imstat @sciflat.lis >> GS-CAL20040613.log 
imstat @sciflat.lis
!echo "--------------------------------------------" >> GS-CAL20040613.log
#skip the first one because the mean is poor
delete flat.lis
gemlist S20040613S 172-180 > flat.lis

# For very short flats, the dark current is negligible and darks
# only add noise.  In this case, a comparison with and without
# subtracting darks reports S/N=551 w/o darks, =512 w/dark-subtraction.
# The two darks are identical to within 0.2%, with no visible 
# differences in structure.
#
# not required to cut longslit data, but it avoids a large number 
# of "bad" pixels in the DQ array
imdel rn@flat.lis
nsreduce n@flat.lis fl_nscut+ fl_nsappwav- fl_sky- fl_flat- 
imdelete rnS20040613S0172_flat,rnS20040613S0172_flat_bpm.pl
nsflat rn@flat.lis darks=""

#-----process arc-----
#its not necessary to flatfield the arc image, but it helps for faint lines.
imdel rn@arc.lis
nsreduce n@arc.lis fl_nscut+ fl_dark- fl_sky- fl_flat+ \
  flatim=rnS20040613S0172_flat
  
#need to increase cradius for 110 resolution (default=5)
#set thresh to avoid matching lines on noise
#NOTE that fl_inter+ does not work in batch mode 
#The parameters here worked automatically for this dataset 
# BUT ALWAYS CHECK THE DISPERSION SOLUTION BEFORE CONTINUING.
# YOU MAY NEED TO RUN THIS STEP MANUALLY.
imdel wrn@arc.lis
nswave rn@arc.lis cradius=20. fl_inter- thresh=400. \
  database="GS-CAL20040613_database" 

#-----trace standard pinholes-------
imdel rn@pinhole.lis
nsreduce n@pinhole.lis fl_nscut+ fl_dark- fl_sky- fl_flat-
# display rnS20040613S0183[sci,1] 1
nssdist rn@pinhole.lis coordlist=gnirs$data/pinholes-short.lis fl_inter-

#-----reduce telluric-----
#check sky range
del phu.lis
!echo "--------------------------------------------" >> GS-CAL20040613.log
gemextn n@telluric.lis proc=expand index=0 > phu.lis
!echo "Telluric Exposure Dates: " >> GS-CAL20040613.log
hselect @phu.lis $I,ut yes >> GS-CAL20040613.log
!echo "--------------------------------------------" >> GS-CAL20040613.log
hselect @phu.lis $I,ut yes
#frames are 17-24 secs apart, use for skyrange in nsreduce

imdel rn@telluric.lis
nsreduce n@telluric.lis fl_nscut+ fl_dark- fl_sky+ skyrange=25 fl_flat+ \
  flatim=rnS20040613S0172_flat 

#transform (use fl_align here, so when we combine all should be aligned)
imdel trn@telluric.lis
nstransform rn@telluric.lis lamp=wrn@arc.lis sdist=rn@pinhole.lis \
  fl_align+

#combine (single group) tellurics
imdel trnS20040613S0161_comb
nscombine trn@telluric.lis

#and extract.
imdel xtrnS20040613S0161_comb
nsextract trnS20040613S0161_comb
#final S/N ~46 per pixel (this is due to excessive coadds and high \
#readnoise readmode)

#delete extra files
delete *lis
delete tmp*
imdel nS20040613S????.fits
imdel rnS20040613S????.fits
imdel trnS20040613S????.fits


