# Copyright(c) 2004-2005 Association of Universities for Research in Astronony, Inc.
#
# GNIRS data reduction script
#
# This data processing was done for data quality assessment by Gemini staff.
# The processing is not designed to give the best possible result.  The user
# of these data is encouraged to use the provided "final" images, and the
# processing steps described here, only as a guide and to re-reduce the
# data carefully to obtain the best possible reduction.
#
# Observation UT date: 2004 Nov 26
# Data filename prefix: S20041126S
#
# Comments from data processor (Claudia Winge):
#
# This is an example of a basic calibration reduction of a cross-dispersed/32 
#   (R~2000) dataset.  The data consist of GCAL flats, an arc, a pinhole and 
#   a telluric standard.  The obslog entries included below (just before the 
#   processing steps) give a brief description of the data.
#
#
# This line points to the raw data 
set rawdir=/net/sabrina/staging5/

# Load required tasks/packages
gemini
gnirs

#-----------setup for gnirs-----------
# set default parameters
# use gnirs defaults except where specified on command lines below
unlearn gemini
unlearn gemtools
unlearn gnirs

# define logfile and database
# to start from scratch, delete existing log file
#delete GS-CAL20041126.log
gnirs.logfile="GS-CAL20041126.log"

# set database dir
gnirs.database="GS-CAL20041126_database"

# to start from scratch, delete database entries if present
#if (access("GS-CAL20041126_database")) delete GS-CAL20041126_database/*

# load default header keywords for gnirs
nsheaders gnirs
set stdimage=imt1024


#Configuration: Short blue camera, XD prism, 32l/mm grating, 0.30 arcsec slit

#GS-CAL20041126-3   0147     23:55    GCALflat   none    XD      LgPinh   32/2.20       SB/SXD    1/1/2    
#GS-CAL20041126-9   0219-222 03:19    HIP3483    ABBA    XD      0.30     32/2.22       SB/SXD    15/1/1 
#GS-CAL20041126-18  0323-333 09:02    GCALflat   none    X       0.30     32/2.22       SB/SXD    1.5/1/4  flats 
#GS-CAL20041126-19  0334     09:09    Ar         none    XD      0.30     32/2.22       SB/SXD    3/1/4    arc 
#GS-CAL20041126-19  0335-349 09:09    GCALflat   none    XD      0.30     32/2.22       SB/SXD    1/1/4    flats  
#GS-CAL20041126-19  0350-362 09:09    GCALflat   none    J       0.30     32/2.22       SB/SXD    1.5/1/4  flats  


#- the combination of XD filter response and IR lamp spectral shape
#- makes it quite hard to get a good illumination for the higher
#- orders without heavily saturating the lower ones. The temporary
#- solution for it is to take three sets of flats, the XD filter 
#- covers orders 3 and 4, the J filter covers order 5 and the 
#- X filter covers order 6. There is no flat for orders 7 and 8
#- The downside on this procedure is that, due to the sharp cut-off
#- of the order separating filters, the flats for orders 5 and 6 do
#- not cover the whole order. For the same reason, a high order spline3
#- function is needed (to follow the spectral shape)

#- Another option we have been pursuing is to get red saturated flats 
#- at the end of the night, and use those to create the flat fields 
#- for the higher orders (6 to 8).
#- It does give a better result than X and covers the blue end of
#- the spectrum. The procedure is exactly the same, and the resulting
#- flats are inserted in positions [10-18]

#- Order 5 still remains a bit of a problem. It is too faint with the
#- IR+XD combination, and saturates with the long QH+XD flats. The J
#- filter does not cover the whole order as well as one would like.
#- The current experiment is to use short QH+XD flats to get more
#- ligth in that order.

#- For any of the above combinations, the procedure is the same:
#-  - run nsflat for each subset of flats. 
#-  - insert the usable orders in the final flat
#- The options are:
#-    1) IR+XD (orders 3+4); QH+J (order 5); QH+X (order 6); orders 7 and 8
#-       are blank
#-    2) IR+XD (orders 3+4); QH+J (order 5); long QH+XD (orders 6 to 8)
#-    3) IR+XD (order 3); short QH+XD (orders 4+5); long QH+XD (orders 6 to 8)

#- Option 3 seems to give the best results, but be aware that the saturated
#- flats are taken at the end of the night (NOT with the science frames),
#- and therefore the grating may have been moved.

#- This example uses option (1)

#- Create lists for XDS Slit 0.3------------
delete *.lis ver- >& dev$null

#- Filter XD 
gemlist S20041126S 335-349 > flat1.lis

#- Filter J 
gemlist S20041126S 350-362 > flat2.lis

#- Filter X 
gemlist S20041126S 323-333 > flat3.lis

gemlist S20041126S 147 > pin.lis
gemlist S20041126S 334 > arc.lis
gemlist S20041126S 219-222 > telluric1.lis

cat flat*.lis > allflats.lis
concat allflats.lis,pin.lis,arc.lis,telluric1.lis > allcal.lis

#-------prepare cals----
#- nprepare . Shift determined after one iteration

#- here is how to do it, if not done before:
#- Choose a well exposed order 3 flat
imdelete S20041126S0335.fits ver- >& dev$null
#- run nsprepare with indef shifts; fl_forc+ required to fix WCS
nsprepare S20041126S0335.fits rawpath=rawdir$ shiftx=INDEF shifty=INDEF \
   fl_chec+ fl_forc+
   
#- and write down the shift it gives. Then use that one for all
#- remaining cals (more specifically, use the same shiftx for flats and
#- corresponding science/telluric frames, to be sure to be flat-
#- fielding the same pixels)

#- shiftx=5.9 (can also be done by specifying shiftimage=<image-with-desired-shift>)

imdelete n@allcal.lis ver- >& dev$null
nsprepare @allcal.lis rawpath=rawdir$ shiftx=5.9 shifty=0 fl_chec+ fl_forc+

#- Note that nsprepare looks for a well exposed order 3 to obtain the shift,
#- so shiftx/y=INDEF will work best with IR+XD flats. 

# ------generate flatfields--------
imdelete rn@allflats.lis ver- >& dev$null
nsreduce n@allflats.lis fl_sky- fl_nscut+ fl_flat- fl_dark- fl_nsappwave- \
    fl_corner+

#- reduce pinhole
imdelete rn@pin.lis ver- >& dev$null
nsreduce n@pin.lis fl_nscut+ fl_nsappwave- fl_dark- fl_sky- fl_flat- fl_corner+

#- Now obtain the S-distortion - used only if transforming science data
#- or if doing the wavelength calibration below

#- When presented with the cross section, just hit "m" on each peak
#- and accept the pixel value it gives. 
#- Do not use the faint peak to the left (it's partially vignetted by
#- the decker)
#- For each order, check if the offset between the peak and the 
#- listed value is consistent for all features, and if the automated 
#- finding does not associate the same value for two features (e.g, 
#- the two rightmost peaks in order 4 in this example)
#- The offset will be different for different orders, and the 
#- way to override a value is to give a "better guess"
#- For example, the peaks in order 4 are 100.7 and 108.8. The task 
#- associated to them both the 104.7 value. The offset is a consistent
#- 4pix between measured and listed, so try entering 112. The task will
#- then correctly associate the 112.9 value to that pinhole.


#- I had to run this from the cl prompt, otherwise weird things happen
#- due to a bug on running IRAF remotely in a Linux box.

#- Note the use of nlost=0 to stop the task where the pinhole ends
#- this is needed to avoid "tracing into the noise" in the higher orders

nssdist rn@pin.lis coordlist=gnirs$data/pinholes-short-dense.lis \
    section="[*,100]" fl_inter+ function="legendre" order=5  minsep=5 \
    thresh=1000 nlost=0.

#- now, the flats: check counts
#- just to make sure there are counts on all frames

!echo "--------------------------------------------" >> GS-CAL20041126.log
delete tmpflat ver- >& dev$null

gemextn rn@flat1.lis proc="expand" extname="SCI" extver="1" > tmpflat
!echo "Order 3-4 Flats: " >> GS-CAL20041126.log
imstatistic @tmpflat >> GS-CAL20041126.log
delete tmpflat ver- >& dev$null

gemextn rn@flat2.lis proc="expand" extname="SCI" extver="3" > tmpflat
!echo "Order 5 flats: " >> GS-CAL20041126.log
imstatistic @tmpflat >> GS-CAL20041126.log
delete tmpflat ver- >& dev$null

gemextn rn@flat3.lis proc="expand" extname="SCI" extver="4" > tmpflat
!echo "Order 6 Flats: " >> GS-CAL20041126.log
imstatistic @tmpflat >> GS-CAL20041126.log
!echo "--------------------------------------------" >> GS-CAL20041126.log
delete tmpflat ver- >& dev$null

#- Flat names: nsflat outputs _flat.fits, but for GSA compatibility
#- only the final flat (which is archived) should have that suffix
#- Until the final method to obtain flats is decided, the following 
#- is being done:
#-  PI still gets the intermediate flats, but we will call them _flat1,
#-    _flat2, flat3
#-  final flat gets named _flat. This is the only one that will be
#-     archived.

#- The 'correct' way is to run the first
#- instance of nsflat interactively, and recenter the aperture to the central
#- pinhole. It may be necessary also to change the lower and upper limits
#- to be sure to include all the iluminated area (+/-20 or 25 works just fine,
#- or use the "l" and "u" keys to custom mark the lower and upper edges
#- of the aperture, respectively)
#- There is no need to do the trace or the flat fits interactively 
#- (although it does not hurt, particularly the fit spectra part)
#-   Find apertures for rnS20041128S0050[SCI,1]?  (yes): <return>
#-   Edit apertures for rnS20041128S0050[SCI,1]?  (yes): <return>
#-    ... recenters and adjust the size of the aperture
#-   Trace apertures for rnS20041128S0050[SCI,1]?  (yes): <return>
#-   Fit traced positions for rnS20041128S0050[SCI,1] interactively?  (yes): no
#-    ... or yes if you want to check the tracing. Legendre order 4-5 works best
#-   Write apertures for rnS20041128S0050[SCI,1] to database?  (yes): <return>
#-   Write apertures for tmp<whatever> to database?  (yes): <return>
#-   Flatten apertures in tmp<whatever>?  (yes): <return>
#-   Fit spectra from tmp<whatever> interactively?  (yes): no
#-    ... or yes if you want to check the normalization fit (see below for
#-        more details on the spline3 function)
                                   
#- Run interactively (should be done at least for the first
#- nsflat call to define the apertures correctly. The default values
#- for the normalization function tend to be too low for orders 5 and 6
#- when using the J and X filter combination so it may be good to run 
#- all three nsflat calls with fl_inter+)

#- Order 3-4 - good counts in orders 3 and 4, fainter but present
#- at the lower part of order 5. Requires spline3>60 for order 3
#- and > 40 for order 4. 

delete rnS20041126S0335_flat1*.* ver- >& dev$null
nsflat rn@flat1.lis trace=rnS20041126S0147 \
    flatfile="rnS20041126S0335_flat1.fits" fl_inter+ fl_corner+ lthresh=10.

nxdisplay rnS20041126S0335_flat1.fits 1

reset stdimage=imt1024

#- Order 5 - good counts in order 5, mostly the top part of it
#- and the lower end of order 6. Requires spline3 >50 for order 5 (to
#- go around the filter edge) and similar if using the order 6 part

delete rnS20041126S0350_flat2*.* ver- >& dev$null
nsflat rn@flat2.lis trace=rnS20041126S0147 \
    flatfile="rnS20041126S0350_flat2.fits" fl_inter+ fl_corner+ lthresh=10.
display rnS20041126S0350_flat2[sci,3] 2

#- Order 6 - good counts in order 6, some light starts peaking up
#- at the top of order 5, and some still shows up at the lower end of
#- order 7. Requires similar spline3 pieces as filter J flats (50 or so)

delete rnS20041126S0323_flat3*.* ver- >& dev$null
nsflat rn@flat3.lis trace=rnS20041126S0147 \
    flatfile="rnS20041126S0323_flat3.fits" fl_inter+ fl_corner+ lthresh=10.
display rnS20041126S0323_flat3[sci,4] 3

#- So now we have three separate files, each one covering part of 
#- the spectral region. To use in nsreduce, need to put all together
#- in one single MEF file, with correct VAR and DQ extensions

#- create the final flat: order: 3-4 flat1 -> position 0-6,
#-                               5 flat2 -> position 7-9
#-                               6 flat3 -> position 10-18

#- if using option (2), flat1(IR+XD) is inserted in positions 0-6; 
#- flat2 (QH+J) in 7-9,  and flat3 (long QH+XD) in 10-18

#- if using option (3), flat1 (IR+XD) is inserted in positions 0-3;
#- flat2 (short QH+XD) in 4-9, and flat3 (long QH+XD) in 10-18

imdelete rnS20041126S0335_flat.fits ver- >& dev$null
fxcopy rnS20041126S0335_flat1.fits rnS20041126S0335_flat.fits "0-6" new_file+
fxinsert rnS20041126S0350_flat2.fits rnS20041126S0335_flat.fits[6] \
    groups="7-9"
fxinsert rnS20041126S0323_flat3.fits rnS20041126S0335_flat.fits[9] \
    groups="10-18"

#- Blank (set to 1) orders 7 and 8 since there is no signal there
#- If using options (2) and (3), this step is not needed (comment
#- the imreplace lines below)

imreplace rnS20041126S0335_flat.fits[sci,5] 1.
imreplace rnS20041126S0335_flat.fits[sci,6] 1.
imreplace rnS20041126S0335_flat.fits[var,5] 0.
imreplace rnS20041126S0335_flat.fits[var,6] 0.
imreplace rnS20041126S0335_flat.fits[dq,5] 0.
imreplace rnS20041126S0335_flat.fits[dq,6] 0.

#- If wavelength calibrating, here is what is needed:

#- Reduce arc (may want to flatfield, which is why it is done here)
#- I found that the flatfield introduces too much noise to be useful

imdelete rn@arc.lis ver- >& dev$null
nsreduce n@arc.lis fl_nscut+ fl_nsappwave- fl_dark- fl_sky- fl_flat- fl_corner+

#- In the two steps below, the NSTRANSFORM task is called from inside 
#- NSWAVELENGTH for S-distortion correction only and then  by itself for 
#- S-distortion and wavelength transformation.

#- In NSWAVELENGTH, the transformation in the spectral direction only (the
#- S-distortion) is used to "extract" an arc along the spectral tracing, 
#- which is then passed to IDENTIFY to obtain the wavelength solution, then to
#- REIDENTIFY to repeat the identification in the full 2D frame.

#- Note that the arc lines are horizontal, NOT perpendicular to the dispersion
#- For the default XD/32 l/mm setting the automatic wavelength identification 
#- is good. Still, in IDENTIFY faint lines must be deleted particularly for 
#- orders 6 to 8 where the noise pattern gets confused with real lines.

#- After that, the mapping along the tracing (obtained from the pinhole) 
#- and the one along the lines (obtained from the arc) are combined using
#- NSTRANSFORM to generate a 2D-map which allows to rectify the spectra
#- and calibrate in wavelength in one step.

#- NSTRANSFORM needs quite a lot of manual intervention:
#- first of all, when runing with fl_inter+ it ignores the yorder value and 
#- reverts to the default value (2) which is too low. As can be seen from the
#- residuals (see transform for a list of keystrokes and corresponding plots), 
#- yorder>4 is needed.

#- Then, bad data points must be deleted. Those are easy to see when fitting the 
#- S-distortion but not as clear with the wavecal one.
#-   Wavecal: plot the y residuals (type xyyrr) and just delete obvious outliers
#-            (residuals larger than 0.5A). Don't forget to hit "f" to redo 
#-            the fit
#-   S-distortion: again, yorder>4 is needed. First go to the x-y plot 
#-                 (type xxyyr).
#- This will present the actual x-y map. It will be quite obvious which points 
#- are bad, particularly in the higher orders, but for starters, delete the 
#- first and last three rows. Type "f" then go and check the residuals (xyyrr).
#- Change the order of the fit if needed (no more than yord=5 and keep xord=2). 
#- For the higher orders (7 and 8) the points in the top will be garbage - 
#- where the trace was lost. Delete all of those.

#- Using yorder<=4 may not be enough to correct the s-distortion well for the 
#- lower orders. Some distortion may still be seen in order 6, and it can be 
#- quite clear in orders 7 and 8, but that should be beyond where the spectra 
#- will go anyway (the fit is being extrapolated, so the transformation can be 
#- anything). In practice, look for tilted arc lines after transformation
#- (the transformed arc should  be straight along columns, and the arc lines 
#- should be horizontal). Unfortunatelly, there is no "canned recipe" for this 
#- step, trial and error is still the only way to find the best solution.
#- If transformation is being applied to science data and spatial information 
#- is important, a more careful fit should be done. Sometimes a *lower* order 
#- (yorder=3) works better for order 8 (where there are fewer points to 
#- constraint the fit)

#- this one does not work for me from inside the script - had to run from 
#- the cl> prompt

imdelete wrn@arc.lis ver- >& dev$null
nswavelength rn@arc.lis fl_inter+ sdist="rnS20041126S0147" threshold=10. \
    nlost=10 fwidth=5.

#- Check to see if it looks ok (also run from terminal)
#- use fl_stripe- to be able to see if the transformation is
#- good. Use the same yorder as when running nswavelength (you want
#- to apply the same S-distortion correction as was used to 

imdelete twrn@arc.lis ver- >& dev$null
nstransform wrn@arc.lis lamptransf="wrnS20041126S0334" \
    sdisttrans="rnS20041126S0147" fl_inter+ fl_stripe- xorder=2 yorder=5

nxdisplay twrnS20041126S0334.fits 1
reset stdimage=imt1024


#- Telluric standard:

#- There are two sky selection parameters in nsreduce, skyrange and nodsize.
#- Skyrange selects the sky based on time - all frames within that time range
#- are valid sky frames for a given dataset. Nodsize uses distance - frames
#- within that distance are NOT valid sky frames for a given dataset. The
#- task uses a combination of both to select a sky, so it will not use a frame
#- in the same position or taken too long before or after the dataset being 
#- processed. It can however decide to combine two or more frames to create a sky
#- Tellurics are usually short exposures so combining frames for creating the
#- sky works well (thus the skyrange=INDEF). For longer exposures, set
#- skyrange to a little over the time between frames (such that only the 
#- next/previous frame will be used for sky subtraction)

#- determining the time between exposures:
!echo "--------------------------------------------" >> GS-CAL20041126.log
delete tmpphu ver- >& dev$null
gemextn n@telluric1.lis proc="expand" index="0" > tmpphu
!echo "Telluric : " >> GS-CAL20041126.log
hselect @tmpphu UT yes >> GS-CAL20041126.log
!echo "--------------------------------------------" >> GS-CAL20041126.log

# ~30 sec

#- reduce tellurics
imdelete rn@telluric1.lis ver- >& dev$null
nsreduce n@telluric1.lis fl_nscut+ fl_corner+ fl_process+ fl_nsapp- \
    fl_sky+ skyrange=30 fl_flat+ flatimage="rnS20041126S0335_flat.fits" \
    nodsize=3.0

#- Combine
imdelete rnS20041126S0219_comb.fits ver- >& dev$null
nscombine rn@telluric1.lis

#- If wavelength calibration exists, apply it here
#- if fl_inter+ run from terminal, otherwise use the same
#- x and y orders here as defined in the test run of 
#- nstranform above. This will not "straighten" the spectra,
#- it will still be tilted/curved in the detector, however it will 
#- "flip" them in y.
#- If full correction is desirable (eg. for extended targets), 
#- add the pinhole to the transformation (sdisttrans="rnS20041126S0147") 
#- and run it interactively to get the best solution (may have to be done
#- more than once, and be prepared for a few surprises, 
#- particularly in the higher orders!)

imdelete trnS20041126S0219_comb.fits ver- >& dev$null
nstransform rnS20041126S0219_comb.fits lamptrans="wrnS20041126S0334.fits" \
    sdisttrans="" fl_inter+ fl_stripe- xorder=2 yorder=5


#- Extract - without the wavelength cal
imdelete xrnS20041126S0219_comb.fits ver- >& dev$null
nsextract rnS20041126S0219_comb.fits line=400 nsum=20 upper=6 low=-6 \
    fl_inter+ fl_apall+ fl_trace+

#- and with it (note that the spectra are now flipped in y so I change
#- the line value where tracing the spatial profile to find the peak)
#- However, be aware that the task resets to default on every order, 
#- so for order 6, it will be centering in noise. Type :line 900 to see
#- the real peak, then shift the aperture to that.

imdelete xtrnS20041126S0219_comb.fits ver- >& dev$null
nsextract trnS20041126S0219_comb.fits line=750 nsum=20 upper=6 low=-6 \
    fl_inter+ fl_apall+ fl_trace+


#- delete extra files
#- keep those lines commented until you are done

#delete *lis
#delete tmp*
#imdel nS20041126S????.fits
#imdel rnS20041126S????.fits
#imdel trnS20041126S????.fits

